#!/usr/bin/env python3

from _libhydrogen import ffi
from _libhydrogen import lib as h

__all__ = [
    # internal
    'hydro_call_init',
    'hydro_version',
    'pyhy_version',
    'dump_keypair_hex',
    # rand
    'hydro_random_u32',
    'hydro_random_uniform',
    'hydro_random_buf',
    'hydro_random_buf_deterministic',
    'hydro_random_ratchet',
    'hydro_random_reseed',
    # hash
    'hydro_hash_keygen',
    'hydro_hash_hash',
    'hydro_hash', # hydro_hash_init, hydro_hash_update, hydro_hash_final
    # kdf
    'hydro_kdf_master_keygen',
    'hydro_kdf_derive_from_key',
    # secretbox
    'hydro_secretbox_keygen',
    'hydro_secretbox_encrypt',
    'hydro_secretbox_decrypt',
    'hydro_secretbox_probe_create',
    'hydro_secretbox_probe_verify',
    # sign
    'hydro_sign',
    'hydro_sign_keygen',
    'hydro_sign_keygen_deterministic',
    # kx
    'hydro_kx_keygen', # all
    'hydro_kx_n_1',  'hydro_kx_n_2', # N
    'hydro_kx_kk_1', 'hydro_kx_kk_2', 'hydro_kx_kk_3', # KK
    'hydro_kx_xx_1', 'hydro_kx_xx_2', 'hydro_kx_xx_3', 'hydro_kx_xx_4', # XX
    # pwhash
    'hydro_pwhash_keygen',
    'hydro_pwhash_deterministic',
    'hydro_pwhash_create',
    'hydro_pwhash_verify',
    'hydro_pwhash_derive_static_key',
    'hydro_pwhash_reencrypt',
    'hydro_pwhash_upgrade',
    # helpers
    'hydro_memzero',
    'hydro_equal',
    'hydro_bin2hex',
    'hydro_hex2bin',
    'hydro_increment',
    'hydro_compare',
    'hydro_pad',
    'hydro_unpad'
]

h.hydro_init()
__version__ =  '0.0.3'

################################################################################
# Internal utilities
################################################################################
def hydro_call_init():
    h.hydro_init()

def hydro_version():
    return 'libhydrogen v%d.%d' % (h.HYDRO_VERSION_MAJOR, h.HYDRO_VERSION_MINOR)

def pyhy_version():
    return 'pyhy v%s' % __version__

def dump_keypair_hex(pair):
    print('\ndump_keypair_hex')
    try:
        print('\tsk', bytes(pair.sk).hex())
        print('\tpk', bytes(pair.pk).hex())
    except Exception as e:
        print('ERROR: keypair must have pk, sk fields')

################################################################################
# rand
################################################################################
__all__ += [ 'hydro_random_SEED' ]
hydro_random_SEED = h.hydro_random_SEEDBYTES

def hydro_random_u32():
    num = h.hydro_random_u32()
    assert num >= 0
    return num

def hydro_random_uniform(ulimit):
    num = h.hydro_random_uniform(ulimit)
    return num

def hydro_random_buf(ct):
    assert ct > 0
    buf = ffi.new('uint8_t[]', ct)
    h.hydro_random_buf(buf, ct)
    return bytes(buf)

def hydro_random_buf_deterministic(ct, seed):
    assert ct > 0
    assert len(seed) == h.hydro_random_SEEDBYTES
    buf = ffi.new('uint8_t[]', ct)
    h.hydro_random_buf_deterministic(buf, ct, seed)
    return bytes(buf)

def hydro_random_ratchet():
    h.hydro_random_ratchet()

def hydro_random_reseed():
    h.hydro_random_reseed()

################################################################################
# hash
################################################################################
__all__ += [ 'hydro_hash_BYTES', 'hydro_hash_BYTES_MAX', 'hydro_hash_BYTES_MIN', 'hydro_hash_CONTEXTBYTES', 'hydro_hash_KEYBYTES' ]
hydro_hash_BYTES        = h.hydro_hash_BYTES
hydro_hash_BYTES_MAX    = h.hydro_hash_BYTES_MAX
hydro_hash_BYTES_MIN    = h.hydro_hash_BYTES_MIN
hydro_hash_CONTEXTBYTES = h.hydro_hash_CONTEXTBYTES
hydro_hash_KEYBYTES     = h.hydro_hash_KEYBYTES
hydro_hash_BYTES        = h.hydro_hash_BYTES

def hydro_hash_keygen():
    buf = ffi.new('uint8_t[]', h.hydro_hash_KEYBYTES)
    h.hydro_hash_keygen(buf)
    return bytes(buf)

# https://github.com/jedisct1/libhydrogen/wiki/Generic-hashing
class hydro_hash(object):
    """wrapper class for hash creation, verification"""
    def __init__(self, ctx, key):
        """Creates a hydro_hash_state object with ctx and key (both required)"""
        assert (type(ctx) == str) and (len(ctx) == 8)
        assert len(key) == hydro_hash_KEYBYTES
        self.st = ffi.new('struct hydro_hash_state *')
        h.hydro_hash_init(self.st, ctx.encode('utf8'), key)

    def update(self, m):
        mlen = len(m)
        # print('update: +%d' % mlen)
        h.hydro_hash_update(self.st, m.encode('utf8'), mlen)

    def final(self):
        """use secret key to generate a signature"""
        buf = ffi.new('uint8_t[]', h.hydro_hash_BYTES)
        h.hydro_hash_final(self.st, buf, h.hydro_hash_BYTES)
        return bytes(buf)

# int
# hydro_hash_hash(uint8_t *out, size_t out_len, const void *in_,
# size_t in_len, const char ctx[hydro_hash_CONTEXTBYTES], const uint8_t *key);
def hydro_hash_hash(outlen, d, ctx, key=None):
    assert (type(ctx) == str) and (len(ctx) == 8)
    assert (outlen >= h.hydro_hash_BYTES_MIN) and (outlen <= hydro_hash_BYTES_MAX)
    dlen = len(d)
    buf = ffi.new('uint8_t[]', outlen)
    if key is None:
        key = ffi.NULL
    if h.hydro_hash_hash(buf, outlen, d.encode('utf8'), dlen, ctx.encode('utf8'), key) == -1:
        return None
    return bytes(buf)

################################################################################
# kdf
################################################################################
__all__ += [ 'hydro_kdf_CONTEXTBYTES', 'hydro_kdf_KEYBYTES', 'hydro_kdf_BYTES_MAX', 'hydro_kdf_BYTES_MIN' ]
hydro_kdf_CONTEXTBYTES  = h.hydro_kdf_CONTEXTBYTES
hydro_kdf_KEYBYTES      = h.hydro_kdf_KEYBYTES
hydro_kdf_BYTES_MAX     = h.hydro_kdf_BYTES_MAX
hydro_kdf_BYTES_MIN     = h.hydro_kdf_BYTES_MIN

def hydro_kdf_master_keygen():
    buf = ffi.new('uint8_t[]', h.hydro_kdf_KEYBYTES)
    h.hydro_kdf_keygen(buf)
    return bytes(buf)

def hydro_kdf_derive_from_key(subkey_len, id, ctx, master_key):
    assert ((subkey_len >= 16) and (subkey_len <= 65535))
    assert (type(ctx) == str) and (len(ctx) == 8)
    buf = ffi.new('uint8_t[]', subkey_len)
    h.hydro_kdf_derive_from_key(buf, subkey_len, id, ctx.encode('utf8'), master_key)
    return bytes(buf)

################################################################################
# Secretbox
################################################################################
__all__ += [ 'hydro_secretbox_CONTEXTBYTES', 'hydro_secretbox_HEADERBYTES', 'hydro_secretbox_KEYBYTES', 'hydro_secretbox_PROBEBYTES' ]
hydro_secretbox_CONTEXTBYTES    = h.hydro_secretbox_CONTEXTBYTES
hydro_secretbox_HEADERBYTES     = h.hydro_secretbox_HEADERBYTES
hydro_secretbox_KEYBYTES        = h.hydro_secretbox_KEYBYTES
hydro_secretbox_PROBEBYTES      = h.hydro_secretbox_PROBEBYTES

def hydro_secretbox_keygen():
    buf = ffi.new('uint8_t[]', h.hydro_secretbox_KEYBYTES)
    h.hydro_secretbox_keygen(buf)
    return bytes(buf)

def hydro_secretbox_encrypt(m, mid, ctx, key):
    assert (type(ctx) == str) and (len(ctx) == 8)
    mlen = len(m)
    buf = ffi.new('uint8_t[]', mlen + h.hydro_secretbox_HEADERBYTES)
    h.hydro_secretbox_encrypt(buf, m.encode('utf8'), mlen, mid, ctx.encode('utf8'), key)
    return bytes(buf)

def hydro_secretbox_decrypt(c, mid, ctx, key):
    assert (type(ctx) == str) and (len(ctx) == 8)
    clen = len(c)
    buf = ffi.new('uint8_t[]', clen - h.hydro_secretbox_HEADERBYTES)
    if h.hydro_secretbox_decrypt(buf, c, clen, mid, ctx.encode('utf8'), key) != 0:
        return None
    return bytes(buf)

def hydro_secretbox_probe_create(c, ctx, key):
    assert (type(ctx) == str) and (len(ctx) == 8)
    clen = len(c)
    buf = ffi.new('uint8_t[]', h.hydro_secretbox_PROBEBYTES)
    h.hydro_secretbox_probe_create(buf, c, clen, ctx.encode('utf8'), key)
    return bytes(buf)

# NOTE/TODO: appears probe verif is not that strict about clen
def hydro_secretbox_probe_verify(p, c, ctx, key):
    assert (type(ctx) == str) and (len(ctx) == 8)
    clen = len(c)
    if (h.hydro_secretbox_probe_verify(p, c, clen, ctx.encode('utf8'), key) == 0):
        return True
    return False

################################################################################
# Sign
################################################################################
__all__ += [ 'hydro_sign_BYTES', 'hydro_sign_CONTEXTBYTES', 'hydro_sign_SEEDBYTES', 'hydro_sign_PUBLICKEYBYTES', 'hydro_sign_SECRETKEYBYTES' ]
hydro_sign_BYTES          = h.hydro_sign_BYTES
hydro_sign_SEEDBYTES      = h.hydro_sign_SEEDBYTES
hydro_sign_CONTEXTBYTES   = h.hydro_sign_CONTEXTBYTES
hydro_sign_PUBLICKEYBYTES = h.hydro_sign_PUBLICKEYBYTES
hydro_sign_SECRETKEYBYTES = h.hydro_sign_SECRETKEYBYTES

def hydro_sign_keygen():
    pair = ffi.new('struct hydro_sign_keypair *')
    h.hydro_sign_keygen(pair)
    return pair

def hydro_sign_keygen_deterministic(seed):
    pair = ffi.new('struct hydro_sign_keypair *')
    h.hydro_sign_keygen_deterministic(pair, seed)
    return pair

class hydro_sign(object):
    """wrapper class for signature creation, verification"""
    def __init__(self, ctx):
        """Creates a hydro_sign_state object with (required) ctx"""
        assert (type(ctx) == str) and (len(ctx) == 8)
        self.st = ffi.new('struct hydro_sign_state *')
        h.hydro_sign_init(self.st, ctx.encode('utf8'))

    def update(self, m):
        mlen = len(m)
        # print('update: +%d' % mlen)
        h.hydro_sign_update(self.st, m.encode('utf8'), mlen)

    def final_create(self, pair):
        """use secret key to generate a signature"""
        buf = ffi.new('uint8_t[]', h.hydro_sign_BYTES)
        h.hydro_sign_final_create(self.st, buf, bytes(pair.sk))
        return bytes(buf)

    def final_verify(self, sig, pair):
        """use public key to verify a signature"""
        result = h.hydro_sign_final_verify(self.st, sig, bytes(pair.pk))
        if result != 0:
            # print('Final verify = %d' % result)
            return False
        return True

################################################################################
# kx
################################################################################
__all__ += [ 'hydro_kx_SESSIONKEYBYTES', 'hydro_kx_PUBLICKEYBYTES', 'hydro_kx_SECRETKEYBYTES', 'hydro_kx_PSKBYTES', 'hydro_kx_SEEDBYTES' ]
hydro_kx_SESSIONKEYBYTES = h.hydro_kx_SESSIONKEYBYTES
hydro_kx_PUBLICKEYBYTES  = h.hydro_kx_PUBLICKEYBYTES
hydro_kx_SECRETKEYBYTES  = h.hydro_kx_SECRETKEYBYTES
hydro_kx_PSKBYTES        = h.hydro_kx_PSKBYTES
hydro_kx_SEEDBYTES       = h.hydro_kx_SEEDBYTES

def hydro_kx_keygen():
    pass

# ----------  N  ---------- #
__all__ += [ 'hydro_kx_N_PACKET1BYTES' ]
hydro_kx_N_PACKET1BYTES = h.hydro_kx_N_PACKET1BYTES

def hydro_kx_n_1():
    pass

def hydro_kx_n_2():
    pass

# ---------- KK ----------- #
__all__ += [ 'hydro_kx_KK_PACKET1BYTES', 'hydro_kx_KK_PACKET2BYTES' ]
hydro_kx_KK_PACKET1BYTES = h.hydro_kx_KK_PACKET1BYTES
hydro_kx_KK_PACKET2BYTES = h.hydro_kx_KK_PACKET2BYTES
def hydro_kx_kk_1():
    pass

def hydro_kx_kk_2():
    pass

def hydro_kx_kk_3():
    pass

# ---------- XX ----------- #
__all__ += [ 'hydro_kx_XX_PACKET1BYTES', 'hydro_kx_XX_PACKET2BYTES', 'hydro_kx_XX_PACKET3BYTES' ]
hydro_kx_XX_PACKET1BYTES = h.hydro_kx_XX_PACKET1BYTES
hydro_kx_XX_PACKET2BYTES = h.hydro_kx_XX_PACKET2BYTES
hydro_kx_XX_PACKET3BYTES = h.hydro_kx_XX_PACKET3BYTES

def hydro_kx_xx_1():
    pass

def hydro_kx_xx_2():
    pass

def hydro_kx_xx_3():
    pass

def hydro_kx_xx_3():
    pass

def hydro_kx_xx_4():
    pass

################################################################################
# pwhash
################################################################################
__all__ += [ 'hydro_pwhash_CONTEXTBYTES', 'hydro_pwhash_MASTERKEYBYTES', 'hydro_pwhash_STOREDBYTES' ]
hydro_pwhash_CONTEXTBYTES = h.hydro_pwhash_CONTEXTBYTES
hydro_pwhash_MASTERKEYBYTES = h.hydro_pwhash_MASTERKEYBYTES
hydro_pwhash_STOREDBYTES = h.hydro_pwhash_STOREDBYTES

def hydro_pwhash_keygen():
    pass

def hydro_pwhash_deterministic():
    pass

def hydro_pwhash_create():
    pass

def hydro_pwhash_verify():
    pass

def hydro_pwhash_derive_static_key():
    pass

def hydro_pwhash_reencrypt():
    pass

def hydro_pwhash_upgrade():
    pass

################################################################################
# helpers
################################################################################
def hydro_memzero():
    pass

def hydro_equal():
    pass

def hydro_bin2hex():
    pass

def hydro_hex2bin():
    pass

def hydro_increment():
    pass

def hydro_compare():
    pass

def hydro_pad():
    pass

def hydro_unpad():
    pass

################################################################################














##### eof



#!/usr/bin/env python3
from __future__ import absolute_import, division, print_function
import os
import cffi

ffibuilder = cffi.FFI()

ffibuilder.set_source(
    "_libhydrogen",
    """#include <hydrogen.h>""",
    libraries=['hydrogen']
)

with open(os.path.join(os.path.dirname(__file__), "pyhy.h")) as f:
    ffibuilder.cdef(f.read())

def _clean():
    for f in os.scandir():
        if f.is_file():
            if f.name.endswith('.o') or f.name.endswith('.so'):
                print('Cleaning ... %s' % f.name)
                os.remove(f.name)

if __name__ == "__main__":
    import os, sys
    _clean()
    ffibuilder.compile()

########



#!/usr/bin/env python
from setuptools import find_packages, setup

NAME = 'pyhy'
VERS = '0.0.3'
PACKAGES = [ 'pyhy' ]
SHORT_DESC = 'bindings for libhydrogen using cffi'
LONG_DESC = """Python bindings for libhydrogen, a small cryptographic library
suited for embedded systems requiring a low footprint. Uses cffi.
"""
KEYWORDS = [ 'cryptography', 'crypto', 'embedded', 'encryption', 'libhydrogen', 'hydrogen' ]
# http://pypi.python.org/pypi?%3Aaction=list_classifiers
CLASSIFIERS = [
    'Development Status :: 2 - Pre-Alpha',
    'License :: OSI Approved :: ISC License (ISCL)',
    'Intended Audience :: Developers',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Topic :: Security',
    'Topic :: Security :: Cryptography',
    'Topic :: Software Development :: Embedded Systems'
]

# https://packaging.python.org/tutorials/distributing-packages/#setup-py
# https://github.com/pypa/sampleproject/blob/master/setup.py
if __name__ == '__main__':
    setup(
        name=NAME,
        version=VERS,
        description=SHORT_DESC,
        long_description=LONG_DESC,
        license='BSD',
        url='https://github.com/someburner/pyhy',
        author='Jeff Hufford',
        author_email='jeffrey92@gmail.com',
        maintainer='Jeff Hufford',
        keywords=KEYWORDS,
        classifiers=CLASSIFIERS,
        python_requires='>=3.4',
        setup_requires=["cffi>=1.0.0"],
        cffi_modules=["./bind.py:ffibuilder"],
        install_requires=["cffi>=1.0.0"],
        include_package_data=True,
        packages=['', 'pyhy'],
        package_data={
            '': [ '*.h' ],
            'pyhy': [ './*.py' ]
        }
    )



#!/usr/bin/env python3

from pyhy import *
import sys
import binascii

TEST_CTX = 'test1234'
FAIL_CTX = '1234test'
INVALID_CTX = '123456789'
STATIC_MASTER_KEY = bytes(b'\x82\xb8\x22\x0b\x8b\xb1\xf3\x2b\x63\x68\x9c\xca\x0f\x73\x86\xc3\x7a\x09\xbf\x76\xbd\x66\xf0\xca\x40\x17\x62\x94\x7a\x93\x92\x22')


################################################################################
# rand - TODO better tests
################################################################################
MAX_U32 = 2**32
MAX_U64 = 2**64

def assert_u32(n):
    assert (n >= 0) and (n < MAX_U32), 'got invalid u32 %d' % n

def test_rand():
    n = hydro_random_u32()
    assert_u32(n)
    assert (n >= 0) and (n < MAX_U32), 'got invalid u32 %d' % n
    for i in range(1, 128):
        n = hydro_random_uniform(i)
        assert_u32(n)
    nbuf = hydro_random_buf( hydro_random_SEED )
    assert len(nbuf) == hydro_random_SEED
    dbuf = hydro_random_buf_deterministic(1234, nbuf)
    assert len(dbuf) == 1234

    # nbuf = hydro_random_buf(i, seed)
    hydro_random_ratchet()
    hydro_random_reseed()

################################################################################
# hash - TODO: min/max outlen for hash_ash
################################################################################
def test_hash():
    hkey = hydro_hash_keygen()
    print('hash_keygen:', hkey.hex())
    hash = hydro_hash_hash(hydro_hash_BYTES*2, 'Arbitrary data to hash', TEST_CTX, hkey)
    print('hash_hash:', hash.hex())
    hash = hydro_hash_hash(hydro_hash_BYTES*2, 'Arbitrary data to hash', TEST_CTX)
    print('hash_hash (no key):', hash.hex())

    h1 = hydro_hash(TEST_CTX, hkey)
    h1.update('some data')
    h1.update('more data')
    h1hash = h1.final()
    # print('h1hash:', h1hash.hex())
    h2 = hydro_hash(TEST_CTX, hkey)
    h2.update('some data')
    h2.update('more data')
    h2hash = h2.final()
    assert h1hash == h2hash
    print('h1hash == h2hash')


################################################################################
# kdf
################################################################################
def test_kdf():
    print('\ntest_kdf')
    master_key = hydro_kdf_master_keygen()
    assert len(master_key) == 32
    bad_master_key = hydro_kdf_master_keygen()
    # print('Master key:', master_key.hex())
    subkey1 = hydro_kdf_derive_from_key(16, 0, TEST_CTX, master_key)
    assert len(subkey1) == 16
    subkey2 = hydro_kdf_derive_from_key(16, 0, TEST_CTX, master_key)
    assert subkey1 == subkey2
    subkey2 = hydro_kdf_derive_from_key(16, 0, FAIL_CTX, master_key)
    assert subkey1 != subkey2
    subkey2 = hydro_kdf_derive_from_key(16, 1, TEST_CTX, master_key)
    assert subkey1 != subkey2
    subkey2 = hydro_kdf_derive_from_key(16, 0, TEST_CTX, bad_master_key)
    assert subkey1 != subkey2
    subkey2 = hydro_kdf_derive_from_key(32, 0, TEST_CTX, master_key)
    assert subkey1 != subkey2
    assert len(subkey2) == 32
    bunch_of_subkeys = []
    for i in range(0, 1024):
        subkey_n = hydro_kdf_derive_from_key(16, i, TEST_CTX, master_key)
        assert subkey_n not in bunch_of_subkeys
        bunch_of_subkeys.append(subkey_n)
    subkey1_again = hydro_kdf_derive_from_key(16, 0, TEST_CTX, master_key)
    assert subkey1 == subkey1_again
    return

################################################################################
# Secretbox
################################################################################
TEST_TEXT = 'This is a test'
FAIL_TEXT = 'This is not a test'

def assert_plaintext(ptxt, check_txt):
    if (ptxt is None) or (ptxt.decode() != check_txt):
        raise Exception('failed to decrypt')
    print('Decrypted plaintext: ', ptxt)

def test_secretbox():
    print('\ntest_secretbox')
    goodkey = hydro_secretbox_keygen()
    # print('Generated key:', key.hex())
    ctxt = hydro_secretbox_encrypt(TEST_TEXT, 0, TEST_CTX, goodkey)
    # print('Ciphertext: ', ctxt.hex())
    #### pass case
    ptxt = hydro_secretbox_decrypt(ctxt, 0, TEST_CTX, goodkey)
    assert_plaintext(ptxt, TEST_TEXT)
    #### some fail cases
    badkey = hydro_secretbox_keygen()
    ptxt = hydro_secretbox_decrypt(ctxt, 0, TEST_CTX, badkey)
    assert ptxt is None, 'ptxt should be None'
    ptxt = hydro_secretbox_decrypt(ctxt, 0, FAIL_CTX, goodkey)
    assert ptxt is None, 'ptxt should be None'
    return

def test_secretbox_probes():
    print('\ntest_secretbox_probes')
    key = hydro_secretbox_keygen()
    ctxt = hydro_secretbox_encrypt('This is a test', 0, TEST_CTX, key)
    probe = hydro_secretbox_probe_create(ctxt, TEST_CTX, key)
    # print('Generated probe:', probe.hex())
    assert hydro_secretbox_probe_verify(probe, ctxt, TEST_CTX, key) == True, 'probe verification failed'
    print('probe verified')

################################################################################
# Sign
################################################################################
def test_signature_pass():
    print('\ntest_signature')
    kp = hydro_sign_keygen()
    ss1 = hydro_sign(TEST_CTX)
    ss1.update('first chunk')
    ss1.update('second chunk')
    sig = ss1.final_create(kp)
    # print('Signature: ', sig.hex())

    ss2 = hydro_sign(TEST_CTX)
    ss2.update('first chunk')
    ss2.update('second chunk')
    assert ss2.final_verify(sig, kp) == True, 'signature verification failed'
    print('OK: signature verified')

def test_signature_fail():
    print('\ntest_signature_fail')
    kp = hydro_sign_keygen()
    ss1 = hydro_sign(TEST_CTX)
    ss1.update('first chunk')
    ss1.update('second chunk')
    sig = ss1.final_create(kp)
    # print('Signature: ', sig.hex())

    ss2 = hydro_sign(TEST_CTX)
    ss2.update('first chunk')
    ss2.update('second chunk')
    ss2.update('third chunk weeeee')
    assert ss2.final_verify(sig, kp) == False, 'signature verification should have failed'
    print('OK: signature verification failed')


################################################################################
# kx - TODO
################################################################################


################################################################################
# other
################################################################################
def test_other():
    # context integrity checks
    try: oops = hydro_sign(INVALID_CTX)
    except Exception as e: print('Bad ctx len assertion ok')
    try: oops = hydro_sign(1234)
    except Exception as e: print('Bad ctx type assertion ok')

################################################################################
# Init
################################################################################

def main():
    # wrapper
    print( hydro_version() )
    print( pyhy_version() )
    # rand
    test_rand()
    # hash
    test_hash()
    # kdf
    test_kdf()
    # secretbox
    test_secretbox()
    test_secretbox_probes()
    # sign
    test_signature_pass()
    test_signature_fail()
    test_other()
    # kx - n
    # kx - kk
    # kx - xx
    return


if __name__ == '__main__':
    main()


sys.exit(0)
